@@ -1,13 +1,22 @@
-import warnings
-from collections import OrderedDict
-import numpy as np
-import copy
+"""Kinetic modeling module
+   Author: Vitor Pereira
+"""
 from mewpy.util.parsing import Arithmetic, build_tree
+from mewpy.util.utilities import AttrDict
+from collections import OrderedDict
+import warnings
+import numpy as np
+from typing import Dict, List, Any
+
 
 class Compartment(object):
-    """ class for modeling compartments. """
+    """ class for modeling compartments."""
 
-    def __init__(self, comp_id, name=None, external=False, size=1.0):
+    def __init__(self,
+                 comp_id: str,
+                 name: str = None,
+                 external: bool = False,
+                 size: float = 1.0):
         """
         Arguments:
             comp_id (str): a valid unique identifier
@@ -31,7 +40,10 @@ class Compartment(object):
 class Metabolite(object):
     """ class for modeling metabolites. """
 
-    def __init__(self, met_id, name=None, compartment=None):
+    def __init__(self,
+                 met_id: str,
+                 name: str = None,
+                 compartment: str = None):
         """
         Arguments:
             met_id (str): a valid unique identifier
@@ -50,7 +62,10 @@ class Metabolite(object):
         return str(self)
 
 
-def calculate_yprime(y, rate, substrates, products, substrate_names):
+def calculate_yprime(y,
+                     rate: np.array,
+                     substrates: List[str],
+                     products: List[str]):
     """
     It takes the numpy array for y_prime,
     and adds or subtracts the amount in rate to all the substrates or products listed
@@ -60,23 +75,21 @@ def calculate_yprime(y, rate, substrates, products, substrate_names):
         rate: the rate calculated by the user made rate equation
         substrates: list of substrates for which rate should be subtracted
         products: list of products for which rate should be added
-        substrate_names: the ordered list of substrate names in the model.\
-            Used to get the position of each substrate or product in y_prime
     Returns:
         y_prime: following the addition or subtraction of rate to the specificed substrates
     """
     y_prime = np.zeros(len(y))
 
     for name in substrates:
-        y_prime[substrate_names.index(name)] -= rate
+        y_prime[name] -= rate
 
     for name in products:
-        y_prime[substrate_names.index(name)] += rate
+        y_prime[name] += rate
 
     return y_prime
 
 
-def check_positive(y_prime):
+def check_positive(y_prime: List[float]):
     """
     Check that substrate values are not negative when they shouldnt be.
     """
@@ -92,27 +105,30 @@ class Rule(object):
     """Base class for kinetic rules.
     """
 
-    def __init__(self, r_id, formula: str, parameters: dict = {}):
+    def __init__(self,
+                 r_id: str,
+                 law: str,
+                 parameters: Dict[str, float] = dict()):
         """Creates a new rule
 
         Args:
             r_id (str): Reaction/rule identifier
-            formula (str): The rule string representation.
+            law (str): The rule string representation.
         """
         self.id = r_id
-        self.formula = formula
+        self.law = law
         self._tree = None
         self.parameters = parameters
 
     @property
     def tree(self):
-        """Parsing tree of the formula.
+        """Parsing tree of the law.
 
         Returns:
             Node: Root node of the parsing tree.
         """
         if not self._tree:
-            self._tree = build_tree(self.formula, Arithmetic)
+            self._tree = build_tree(self.law, Arithmetic)
         return self._tree
 
     def parse_parameters(self):
@@ -123,17 +139,30 @@ class Rule(object):
         """
         return list(self.tree.get_parameters())
 
+    def rename_parameter(self, old_parameter: str, new_parameter: str):
+        """Need to rename in the law, tree and parameters"""
+
+        if old_parameter in self.parse_parameters():
+            t = self.tree.replace({old_parameter: new_parameter})
+            self.law = t.to_infix()
+            self._tree = None
+        try:
+            self.parameters[new_parameter] = self.parameters[old_parameter]
+            del self.parameters[old_parameter]
+        except:
+            pass
+
     def get_parameters(self):
         return self.parameters
 
-    def replace(self, parameters=None, local=True, infix=True):
+    def replace(self, parameters: Dict[str, Any] = None, local=True, infix=True):
         """Replaces parameters with values taken from a dictionary.
         If no parameter are given for replacement, returns the string representation of the rule
         built from the parsing tree.
 
         Args:
             parameters (dict, optional): Replacement dictionary. Defaults to None.
-
+            local (bool, optional): use parameter values defined in the rule 
         Returns:
             str: the kinetic rule.
         """
@@ -148,9 +177,10 @@ class Rule(object):
 
     def calculate_rate(self, substrates={}, parameters={}):
         param = dict()
+        param.update(self.parameters)
         param.update(substrates)
         param.update(parameters)
-        param.update(self.parameters)
+
         if len(param.keys()) != len(self.parse_parameters()):
             s = set(self.parse_parameters())-set(param.keys())
             raise ValueError(f"Values missing for parameters: {s}")
@@ -159,28 +189,39 @@ class Rule(object):
         return rate
 
     def __str__(self):
-        return self.formula
+        return self.law.replace(' ', '')
 
     def __repr__(self):
-        return self.replace()
+        return self.replace().replace(' ', '')
 
 
 class KineticReaction(Rule):
 
-    def __init__(self, r_id, formula: str, stoichiometry: dict = {}, parameters: dict = {}, modifiers: list = []):
-        """[summary]
+    def __init__(self,
+                 r_id: str,
+                 law: str,
+                 name: str = None,
+                 stoichiometry: dict = {},
+                 parameters: dict = {},
+                 modifiers: list = [],
+                 reversible: bool = True):
+        """Kinetic reaction rule.
 
         Args:
             r_id (str): Reaction identifier
-            formula (str): kinetic law
+            law (str): kinetic law
+            stoichiometry (dict): The stoichiometry of the reaction.
             parameters (dict, optional): local parameters. Defaults to dict().
             substrates (list, optional): substrates. Defaults to [].
             products (list, optional): products. Defaults to [].
         """
-        super(KineticReaction, self).__init__(r_id, formula, parameters)
+        super(KineticReaction, self).__init__(r_id, law, parameters)
+        self.name = name if name else r_id
         self.stoichiometry = stoichiometry
         self.modifiers = modifiers
         self.parameter_distributions = {}
+        self.reversible = reversible
+        self._model = None
 
     @property
     def substrates(self):
@@ -190,8 +231,28 @@ class KineticReaction(Rule):
     def products(self):
         return [k for k, v in self.stoichiometry.items() if v > 0]
 
+    def rename_parameter(self, old_parameter, new_parameter):
+        super().rename_parameter(old_parameter, new_parameter)
+
+        if old_parameter in self.modifiers:
+            self.modifiers.append(new_parameter)
+            self.modifiers.remove(old_parameter)
+
+        if old_parameter in self.parameter_distributions:
+            self.parameter_distributions[new_parameter] = self.parameter_distributions[old_parameter]
+            del self.parameter_distributions[old_parameter]
+
+    def set_parameter_distribution(self, param, dist):
+        self.parameter_distributions[param] = dist
+
+    def sample_parameter(self, param):
+        dist = self.parameter_distributions.get(param, None)
+        if not dist:
+            raise ValueError(f"The parameter {param} has no associated distribution.")
+        return dist.rvs()
+
     def parse_law(self, map: dict, local=True):
-        """Auxialiary method invoked by the model to build the ODE system.
+        """Auxiliary method invoked by the model to build the ODE system.
 
         Args:
             map (dict): Dictionary of global paramameters replacements.
@@ -199,19 +260,26 @@ class KineticReaction(Rule):
         Returns:
             str: kinetic rule.
         """
-        m = {p_id: f"p['{self.id}_{p_id}']" for p_id in self.parameters.keys()}
+        m = {p_id: f"p['{p_id}']" for p_id in self.parameters.keys()}
         r_map = map.copy()
         r_map.update(m)
 
         return self.replace(r_map, local=local)
 
     def calculate_rate(self, substrates={}, parameters={}):
-        param = {}
+
+        param = dict()
+        # sets model defaults
+        param.update(self._model.get_concentrations())
+        param.update(self._model.get_parameters())
+        # set reaction defaults
+        param.update(self.parameters)
+        # user defined
         param.update(substrates)
         param.update(parameters)
-        param.update(self.parameters)
-        if len(param.keys()) != len(self.parse_parameters()):
-            s = set(self.parse_parameters())-set(param.keys())
+        s = set(self.parse_parameters())-set(param.keys())
+        if s:
+            # check for missing parameters distributions
             r = s - set(self.parameter_distributions.keys())
             if r:
                 raise ValueError(f"Missing values or distribuitions for parameters: {r}")
@@ -224,31 +292,22 @@ class KineticReaction(Rule):
 
     def reaction(self, y, substrate_names, parameter_dict):
 
-        if self.substrate_indexes == []:
-            # need to move this to the model
-            self.get_indexes(substrate_names)
-
-        if self.run_model_parameters == []:
-            self.run_model_parameters = self.get_parameters(parameter_dict)
-
         for modifier in self.modifiers:
-            if modifier.substrate_indexes == []:
-                modifier.get_substrate_indexes(self.reaction_substrate_names)
-            if modifier.parameter_indexes == []:
-                modifier.get_parameter_indexes(self.parameter_names)
+            pass
 
-        substrates = self.get_substrates(y)
-        parameters = copy.copy(self.run_model_parameters)
+        parameters = self.get_parameters(parameter_dict)
+        substrates = self.substrates
 
         if len(self.modifiers) != 0:
-            substrates, parameters = self.calculate_modifiers(substrates, parameters)
+            # substrates, parameters = self.calculate_modifiers(substrates, parameters)
+            pass
 
         rate = self.calculate_rate(substrates, parameters)
 
         y_prime = calculate_yprime(y, rate, self.substrates, self.products, substrate_names)
-        y_prime = self.modify_product(y_prime, substrate_names)
+        # y_prime = self.modify_product(y_prime, substrate_names)
 
-        if self.check_positive:
+        if not self.reversible:
             y_prime = check_positive(y_prime)
 
         return y_prime
@@ -285,11 +344,9 @@ class ODEModel:
 
         self._func_str = None
         self._constants = None
-
         self._m_r_lookup = None
 
     def _clear_temp(self):
-        self.update()
         self._func_str = None
 
     def add_compartment(self, compartment, replace=True):
@@ -313,10 +370,113 @@ class ODEModel:
             raise RuntimeError(f"Metabolite {metabolite.id} already exists.")
 
         if metabolite.compartment not in self.compartments:
-            raise RuntimeError(f"Metabolite {metabolite.id} has invalid compartment {metabolite.compartment}.")
+            raise RuntimeError(f"Metabolite {metabolite.id} \
+                has invalid compartment {metabolite.compartment}.")
 
         self.metabolites[metabolite.id] = metabolite
 
+    @property
+    def reactions(self):
+        return AttrDict(self.ratelaws)
+
+    def get_reaction(self, r_id):
+        if r_id not in self.ratelaws:
+            raise ValueError(f'Unknown reaction {r_id}')
+        r = self.ratelaws[r_id]
+        d = {'id': r_id,
+             'name': r.name,
+             'stoichiometry': r.stoichiometry,
+             'law': r.law,
+             'reversible': r.reversible,
+             'parameters': r.parameters,
+             'modifiers': r.modifiers
+             }
+        return AttrDict(d)
+
+    def get_metabolite(self, m_id):
+        if m_id not in self.metabolites:
+            raise ValueError(f'Unknown metabolite {m_id}')
+        d = {'id': m_id,
+             'name': self.metabolites[m_id].name,
+             'compartment': self.metabolites[m_id].compartment,
+             'formula': self.metabolites[m_id].metadata.get('FORMULA', ''),
+             'charge': self.metabolites[m_id].metadata.get('CHARGE', ''),
+             'y0': self.concentrations[m_id]
+             }
+        return AttrDict(d)
+
+    def find(self, pattern=None, sort=False):
+        """A user friendly method to find reactionsin the model.
+
+        :param pattern: The pattern which can be a regular expression, 
+            defaults to None in which case all entries are listed.
+        :type pattern: str, optional
+        :param sort: if the search results should be sorted, defaults to False
+        :type sort: bool, optional
+        :return: the search results
+        :rtype: pandas dataframe
+        """
+        values = list(self.reactions.keys())
+        if pattern:
+            import re
+            if isinstance(pattern, list):
+                patt = '|'.join(pattern)
+                re_expr = re.compile(patt)
+            else:
+                re_expr = re.compile(pattern)
+            values = [x for x in values if re_expr.search(x) is not None]
+        if sort:
+            values.sort()
+
+        import pandas as pd
+        data = [self.get_reaction(x) for x in values]
+
+        if data:
+            df = pd.DataFrame(data)
+            df = df.set_index(df.columns[0])
+        else:
+            df = pd.DataFrame()
+        return df
+
+    def find_reactions(self, pattern=None, sort=False):
+        return self.find(pattern, sort)
+
+    def find_metabolites(self, pattern=None, sort=False):
+        """A user friendly method to find metabolites in the model.
+
+        :param pattern: The pattern which can be a regular expression, 
+            defaults to None in which case all entries are listed.
+        :type pattern: str, optional
+        :param sort: if the search results should be sorted, defaults to False
+        :type sort: bool, optional
+        :return: the search results
+        :rtype: pandas dataframe
+        """
+        values = list(self.metabolites.keys())
+        if pattern:
+            import re
+            if isinstance(pattern, list):
+                patt = '|'.join(pattern)
+                re_expr = re.compile(patt)
+            else:
+                re_expr = re.compile(pattern)
+            values = [x for x in values if re_expr.search(x) is not None]
+        if sort:
+            values.sort()
+
+        import pandas as pd
+        data = [self.get_metabolite(x) for x in values]
+
+        if data:
+            df = pd.DataFrame(data)
+            df = df.set_index(df.columns[0])
+        else:
+            df = pd.DataFrame()
+        return df
+
+    def get_concentrations(self):
+        return self.concentrations
+
     def set_concentration(self, m_id: str, concentration: float):
         """Sets a metabolite initial concentration
 
@@ -336,6 +496,7 @@ class ODEModel:
             r_id (str): Reaction Identifier
             law (KineticReaction): The reaction rate law.
         """
+        law._model = self
         self.ratelaws[r_id] = law
 
     def get_ratelaw(self, r_id):
@@ -366,7 +527,12 @@ class ODEModel:
 
         for r_id, law in self.ratelaws.items():
             for p_id, value in law.parameters.items():
-                full_id = f"{r_id}_{p_id}"
+
+                full_id = f"{p_id}"
+                if full_id in constants:
+                    warnings.warn(f'renaming {p_id} to {r_id}_{p_id}')
+                    full_id = f"{r_id}_{p_id}"
+                    law.rename_parameter(f"{p_id}", f"{r_id}_{p_id}")
                 constants[full_id] = value
 
         self._constants = constants
@@ -391,13 +557,20 @@ class ODEModel:
             v = coeff * f if coeff > 0 else coeff
             terms.append(f"{v:+g} * r['{r_id}']")
 
-        if f == 0 or len(terms) == 0 or (self.metabolites[m_id].constant and self.metabolites[m_id].boundary):
+        if (f == 0 or
+            len(terms) == 0 or
+            (self.metabolites[m_id].constant and
+             self.metabolites[m_id].boundary
+             )):
             expr = "0"
         else:
             expr = f"1/p['{c_id}'] * ({' '.join(terms)})"
         return expr
 
     def get_parameters(self, exclude_compartments=False):
+        """Returns a dictionary of the model parameters
+
+        """
         if not self._constants:
             self.merge_constants()
         parameters = self._constants.copy()
@@ -406,25 +579,83 @@ class ODEModel:
                 del parameters[c_id]
         return parameters
 
+    def find_parameters(self, pattern=None, sort=False):
+        """A user friendly method to find parameters in the model.
+
+        :param pattern: The pattern which can be a regular expression, 
+            defaults to None in which case all entries are listed.
+        :type pattern: str, optional
+        :param sort: if the search results should be sorted, defaults to False
+        :type sort: bool, optional
+        :return: the search results
+        :rtype: pandas dataframe
+        """
+        params = self.get_parameters()
+        values = list(params.keys())
+        if pattern:
+            import re
+            if isinstance(pattern, list):
+                patt = '|'.join(pattern)
+                re_expr = re.compile(patt)
+            else:
+                re_expr = re.compile(pattern)
+            values = [x for x in values if re_expr.search(x) is not None]
+        if sort:
+            values.sort()
+
+        import pandas as pd
+        data = [(x, params[x]) for x in values]
+
+        if data:
+            df = pd.DataFrame(data, columns=['Parameter', 'Value'])
+            df = df.set_index(df.columns[0])
+        else:
+            df = pd.DataFrame()
+        return df
+
     def deriv(self, y, t):
+        """
+        deriv function called by integrate to be used by dFBA
+
+        For each step when the model is run, the rate for each reaction is calculated
+        and changes in substrates and products calculated.
+        These are returned by this function as y_prime, which are added to y which is 
+        returned by run_model
+
+        Args:
+            y (list): ordered list of substrate values at this current timepoint. 
+               Has the same order as self.run_model_species_names
+            t (): time, not used in this function but required for some reason
+
+        Returns:
+            y_prime - ordered list the same as y, y_prime is the new set of y's for this timepoint.
+        """
         m_y = OrderedDict(zip(self.metabolites, y))
         yprime = np.zeros(len(y))
         for _, reaction in self.ratelaws.items():
             yprime += reaction.reaction(m_y, self.get_parameters())
         return yprime
 
-    def build_ode(self, factors=None, local=False):
+    def build_ode(self, factors: dict = None, local: bool = False) -> str:
+        """ 
+        Axiliary function to build the ODE as a string
+        to be evaluated by eval. Allows the inclusion of factors to
+        be applied to the parameters
+
+        Args:
+            factors (dict): factors to be applied to parameters
+            local (bool): enforces the usage of parameter values defined within
+            the reactions
+        Returns:
+            func_str the right-hand side of the system. 
+        """
 
-        rmap = OrderedDict()
         m = {m_id: f"x[{i}]" for i, m_id in enumerate(self.metabolites)}
         c = {c_id: f"p['{c_id}']" for c_id in self.compartments}
         p = {p_id: f"p['{p_id}']" for p_id in self.constant_params}
         v = {p_id: f"v['{p_id}']" for p_id in self.variable_params}
-        rmap.update(m)
-        rmap.update(c)
-        rmap.update(p)
-        rmap.update(v)
-        
+        rmap = OrderedDict({**m, **c, **p, **v})
+
         parsed_rates = {r_id: ratelaw.parse_law(rmap, local=local)
                         for r_id, ratelaw in self.ratelaws.items()}
 
@@ -459,14 +690,14 @@ class ODEModel:
             p.update(params)
 
         if factors is not None:
-            for k,v in factors.items():
+            for k, v in factors.items():
                 if k in p.keys():
-                    p[k]= v * p[k] 
+                    p[k] = v * p[k]
 
         v = self.variable_params.copy()
         r = r_dict if r_dict is not None else dict()
-        
-        np.seterr(divide='ignore', invalid='ignore')         
+
+        np.seterr(divide='ignore', invalid='ignore')
         exec(self.build_ode(factors), globals())
         ode_func = eval('ode_func')
 
