@@ -58,5 +58,5 @@ class ODEStatus(Enum):
 class ODESolver(ABC):
 
     @abstractmethod
-    def solve(self, y0, t_span, **kwargs):
+    def solve(self, y0, t_points, **kwargs):
         raise NotImplementedError
