@@ -22,7 +22,8 @@ class ScipySolver(ODESolver):
     def set_initial_condition(self, initial_condition):
         self.initial_condition = initial_condition
 
-    def solve(self, y0, t_span, **kwargs):
-        sol = solve_ivp(self.func, t_span, y0, method=methods[self.method],**kwargs)
+    def solve(self, y0, t_points, **kwargs):
+        t_span=[t_points[0],t_points[-1]]
+        sol = solve_ivp(self.func, t_span, y0, method=methods[self.method], t_eval=t_points,**kwargs)
         C = [c[-1] for c in sol.y]
         return C, sol.t, sol.y
